#include <stdio.h>
#include <errno.h>
#include <string.h>

#define MODNAME_MAX_LEN 60
#define NUM_COLUMNS     8

static int file2src(const char* const progname, const char* const binpath, const char* const modname);

int main(int argc, char* argv[])
{
	if(3!=argc)
	{
		fprintf(stderr, "Usage: %s BIN_PATH MODULE_NAME\n", argv[0]);
		return 1;
	}
	else
	{
		const char* const binpath = argv[1];
		const char* const modname = argv[2];

		if(strlen(modname)>MODNAME_MAX_LEN)
		{
			fprintf(stderr, "%s: module name cannot exceed %i characters\n", argv[0], MODNAME_MAX_LEN);
			return 1;
		}

		return file2src(argv[0], binpath, modname);
	}

	return 0;
}

static int file2src(const char* const progname, const char* const binpath, const char* const modname)
{
	FILE* const   infile  = fopen(binpath, "rb");
	FILE*         outfile = NULL;
	unsigned long size    = 0;
	unsigned long column  = NUM_COLUMNS;
	unsigned long part;
	unsigned char buff[256];
	char          outname[MODNAME_MAX_LEN+4];

	if(NULL==infile)
	{
		const int err = errno;
		fprintf(stderr, "%s: cannot open file '%s': %s\n", progname, binpath, strerror(err));
		return 1;
	}

	/* generate C file */

	strcpy(outname, modname);
	strcat(outname, ".c");

	if(NULL==(outfile=fopen(outname, "wb")))
	{
		const int err = errno;
		fclose(infile);
		fprintf(stderr, "%s: cannot open file '%s': %s\n", progname, outname, strerror(err));
		return 1;
	}

	fprintf(outfile, "const unsigned char %s_data[] =\n{", modname);

	while(0<(part=fread(buff, sizeof(char), sizeof(buff)/sizeof(buff[0]), infile)))
	{
		unsigned long i;

		for(i=0;i<part;++size,++column,++i)
		{
			if(0<size)
			{
				fputc(',', outfile);
			}

			if(0<column)
			{
				fputc(' ', outfile);
			}

			if(NUM_COLUMNS==column)
			{
				fputs("\n\t", outfile);
				column = 0;
			}

			fprintf(outfile, "0x%02X", buff[i]);
		}
	}

	if(0!=column)
	{
		fputc('\n', outfile);
	}

	fputs("};\n", outfile);

	fclose(outfile);
	fclose(infile);

	/* generate H file */

	strcpy(outname, modname);
	strcat(outname, ".h");

	if(NULL==(outfile=fopen(outname, "wb")))
	{
		const int err = errno;
		fprintf(stderr, "%s: cannot open file '%s': %s\n", progname, outname, strerror(err));
		return 1;
	}

	fprintf(outfile, "#ifndef FILE2SRC_MODULE_%s_H\n", modname);
	fprintf(outfile, "#define FILE2SRC_MODULE_%s_H\n", modname);
	fprintf(outfile, "const unsigned char %s_data[%lu];\n", modname, size);
	fprintf(outfile, "#endif /* FILE2SRC_MODULE_%s_H */\n", modname);

	fclose(outfile);

	return 0;
}
