/* $Id: ast_datatype.cpp 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "ast_datatype.h"
#include "ast_datatypecons.h"
#include "../astvisitor.h"
#include "wtruntime/types.h"

wtast::DataType::~DataType()
{
	while(!mConses.empty())
	{
		delete mConses.front();
		mConses.pop_front();
	}
}

void wtast::DataType::Accept(AstVisitor* vis)
{
	if(vis->Visit(this))
	{
		for(conses_t::iterator it=mConses.begin(),end=mConses.end();it!=end;++it)
		{
			(*it)->Accept(vis);
		}

		vis->EndVisit(this);
	}
}

wtast::DataType* wtast::DataType::Copy()
{
	DataType* datatype = new DataType(mName, mTypeId);

	for(conses_t::iterator it=mConses.begin(),end=mConses.end();it!=end;++it)
	{
		datatype->AddCons((*it)->Copy());
	}

	if(mDefaultCons)
	{
		datatype->mDefaultCons = datatype->GetCons(mDefaultCons->GetName());
	}

	return datatype;
}

wt_u32 wtast::DataType::GetLoadSize() const
{
	return sizeof(wt_vmptr);
}

bool wtast::DataType::HasFunctionType() const
{
	for(conses_t::const_iterator it=mConses.begin(),end=mConses.end();it!=end;++it)
	{
		if((*it)->HasFunctionType(this))
		{
			return true;
		}
	}

	return false;
}

bool wtast::DataType::IsCopyable() const
{
	for(conses_t::const_iterator it=mConses.begin(),end=mConses.end();it!=end;++it)
	{
		if(!(*it)->IsCopyable(this))
		{
			return false;
		}
	}

	return true;
}

void wtast::DataType::AddCons(wtast::DataTypeCons* cons)
{
	if(mConsMap.find(cons->GetName())==mConsMap.end())
	{
		cons->SetConsId(mNextConsId++);

		mConses.push_back(cons);
		mConsMap[cons->GetName()] = cons;
	}
}
