/* $Id: ast_datatypecons.cpp 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "ast_datatypecons.h"
#include "ast_datatype.h"
#include "ast_type.h"
#include "ast_seqtype.h"
#include "../astvisitor.h"

wtast::DataTypeCons::~DataTypeCons()
{
}

void wtast::DataTypeCons::Accept(AstVisitor* vis)
{
	if(vis->Visit(this))
	{
		//for(types_t::iterator it=mTypes.begin(),end=mTypes.end();it!=end;++it)
		//{
		//	(*it)->Accept(vis);
		//}

		vis->EndVisit(this);
	}
}

void wtast::DataTypeCons::AddTypeBack(Type* type)
{
	if(!type->IsSimultaneous())
	{
		mOffsets.push_back(mDataSize);
		mDataSize += type->GetLoadSize();
		mTypes.push_back(type);
	}
}

bool wtast::DataTypeCons::HasFunctionType() const
{
	for(types_t::const_iterator it=mTypes.begin(),end=mTypes.end();it!=end;++it)
	{
		if((*it)->HasFunctionType())
		{
			return true;
		}
	}

	return false;
}

bool wtast::DataTypeCons::HasFunctionType(const wtast::DataType* dt) const
{
	for(types_t::const_iterator it=mTypes.begin(),end=mTypes.end();it!=end;++it)
	{
		if((*it)->IsSequence() && dt->GetTypeId()==static_cast<wtast::SeqType*>(*it)->GetElementType()->GetTypeId())
		{
			continue;
		}

		if(dt->GetTypeId()!=(*it)->GetTypeId() && (*it)->HasFunctionType())
		{
			return true;
		}
	}

	return false;
}

bool wtast::DataTypeCons::IsCopyable() const
{
	for(types_t::const_iterator it=mTypes.begin(),end=mTypes.end();it!=end;++it)
	{
		if(!(*it)->IsCopyable())
		{
			return false;
		}
	}

	return true;
}

bool wtast::DataTypeCons::IsCopyable(const wtast::DataType* dt) const
{
	for(types_t::const_iterator it=mTypes.begin(),end=mTypes.end();it!=end;++it)
	{
		if((*it)->IsSequence() && dt->GetTypeId()==static_cast<wtast::SeqType*>(*it)->GetElementType()->GetTypeId())
		{
			continue;
		}

		if(dt->GetTypeId()!=(*it)->GetTypeId() && !(*it)->IsCopyable())
		{
			return false;
		}
	}

	return true;
}

wtast::DataTypeCons* wtast::DataTypeCons::Copy()
{
	DataTypeCons* cons = new DataTypeCons(mName);
	cons->mConsId = mConsId;

	for(types_t::iterator it=mTypes.begin(),end=mTypes.end();it!=end;++it)
	{
		cons->AddTypeBack(*it);
	}

	return cons;
}
