/* $Id: ast_declareblock.h 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_DECLAREBLOCK_H
#define WTPARSER_AST_DECLAREBLOCK_H

#include "../astnode.h"
#include "ast_entity.h"
#include <deque>

namespace wtast
{

class Declaration;

class DeclareBlock : public Entity
{
	typedef std::deque<Declaration*> declarations_t;

	declarations_t mDeclarations;
	bool           mImport;
	bool           mExport;

public:
	DeclareBlock(bool imp, bool exp) : Entity(), mImport(imp), mExport(exp)
	{
		// ok
	}

	~DeclareBlock();

	void Accept(AstVisitor* vis);

	DeclareBlock* Copy();

	bool IsImport() const
	{
		return mImport;
	}

	bool IsExport() const
	{
		return mExport;
	}

	void AddDeclarationFront(Declaration* decl)
	{
		mDeclarations.push_front(decl);
	}

	void AddDeclarationBack(Declaration* decl)
	{
		mDeclarations.push_back(decl);
	}

	template<typename Iter>
	void InsertBack(const Iter & first, const Iter & last)
	{
		mDeclarations.insert(mDeclarations.end(), first, last);
	}

	std::size_t GetNumDeclarations() const
	{
		return mDeclarations.size();
	}

	Declaration* GetDeclaration(std::size_t idx)
	{
		if(idx<mDeclarations.size())
		{
			return mDeclarations[idx];
		}
		else
		{
			return 0;
		}
	}

	Kind GetKind() const
	{
		return DECLAREBLOCK;
	}
};

} // namespace wtast

#endif // WTPARSER_AST_DECLAREBLOCK_H

