/* $Id: ast_foreachstatement.h 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_FOREACHSTATEMENT_H
#define WTPARSER_AST_FOREACHSTATEMENT_H

#include "../astnode.h"
#include "ast_statement.h"
#include <string>

namespace wtast
{

class VariableDeclaration;
class Expression;
class Compound;

class ForeachStatement : public Statement
{
	VariableDeclaration* mVariable;
	Expression*          mFrom;
	Expression*          mTo;
	Expression*          mContainer;
	Expression*          mWhile;
	Expression*          mWhich;
	Compound*            mBody;
	bool                 mInclusive;
	bool                 mReverse;
	bool                 mRangeBased;

public:
	ForeachStatement(VariableDeclaration* var, Expression* from, Expression* to, Expression* whil,
		Expression* whic, Compound* body, bool inclusive, bool reverse)
		: Statement()
		, mVariable(var)
		, mFrom(from)
		, mTo(to)
		, mContainer(0)
		, mWhile(whil)
		, mWhich(whic)
		, mBody(body)
		, mInclusive(inclusive)
		, mReverse(reverse)
		, mRangeBased(true)
	{
		// ok
	}

	ForeachStatement(VariableDeclaration* var, Expression* container, Expression* whil,
		Expression* whic, Compound* body, bool reverse)
		: Statement()
		, mVariable(var)
		, mFrom(0)
		, mTo(0)
		, mContainer(container)
		, mWhile(whil)
		, mWhich(whic)
		, mBody(body)
		, mInclusive(true)
		, mReverse(reverse)
		, mRangeBased(false)
	{
		// ok
	}

	~ForeachStatement();

	void Accept(AstVisitor* vis);

	ForeachStatement* Copy();

	Kind GetKind() const
	{
		return FOREACHSTATEMENT;
	}

	VariableDeclaration* GetVariable()
	{
		return mVariable;
	}

	Expression* GetFrom()
	{
		return mFrom;
	}

	Expression* GetTo()
	{
		return mTo;
	}

	Expression* GetContainer()
	{
		return mContainer;
	}

	Expression* GetWhile()
	{
		return mWhile;
	}

	Expression* GetWhich()
	{
		return mWhich;
	}

	Compound* GetBody()
	{
		return mBody;
	}

	bool IsInclusive() const
	{
		return mInclusive;
	}

	bool IsReverse() const
	{
		return mReverse;
	}

	bool IsRangeBased() const
	{
		return mRangeBased;
	}
};

} // namespace wtast

#endif // WTPARSER_AST_FOREACHSTATEMENT_H
