/* $Id: ast_globalblock.h 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_GLOBALBLOCK_H
#define WTPARSER_AST_GLOBALBLOCK_H

#include "../astnode.h"
#include "ast_entity.h"
#include <deque>

namespace wtast
{

class VariableDeclaration;

class GlobalBlock : public Entity
{
	typedef std::deque<VariableDeclaration*> vardecls_t;

	vardecls_t mVarDecls;

public:
	GlobalBlock()
	{
		// ok
	}

	template<typename Iter>
	GlobalBlock(Iter first, Iter last) : mVarDecls(first, last)
	{
		// ok
	}

	~GlobalBlock();

	void Accept(AstVisitor* vis);

	GlobalBlock* Copy();

	std::size_t GetNumVariableDeclarations() const
	{
		return mVarDecls.size();
	}

	VariableDeclaration* GetVariableDeclaration(std::size_t idx)
	{
		if(idx<mVarDecls.size())
		{
			return mVarDecls[idx];
		}
		else
		{
			return 0;
		}
	}

	Kind GetKind() const
	{
		return GLOBALBLOCK;
	}
};

} // namespace wtast

#endif // WTPARSER_AST_GLOBALBLOCK_H
