#include "config_private.h"
#include "funcsig.h"
#include "profile.h"
#include "program.h"
#include "byteorder.h"
#include <string.h>
#include <stddef.h>

struct wtr_funcsig* wtr_funcsig_new(struct wtr_profile* prof, const char* name, wt_u32 argc, const wt_u32* argv, wt_u32 retc, const wt_u32* retv, wt_bool pure, struct wtr_program* prog, wt_bo bo)
{
	const wt_u32              nl  = strlen(name) + 1;
	wt_u32                    ss  = sizeof(struct wtr_funcsig) + nl + sizeof(wt_u32) * (argc + retc);
	struct wtr_funcsig* const fs  = (struct wtr_funcsig*)wtr_profile_sys_malloc(prof, ss);
	char*                     ptr = (char*)(fs+1);
	wt_u32*                   p;
	wt_u32                    i;

	if(NULL==fs)
	{
		return NULL;
	}

	fs->argc = argc;
	fs->retc = retc;
	fs->pure = pure;

	/* copy argument types with byte order swap and resolutions */

	fs->argv = p = (wt_u32*)ptr;
	for(i=0;i<argc;++p,++argv,++i)
	{
		*p = wtr_botoh32(*argv, bo);
		if(!wtr_program_resolvetype(prog, p))
		{
			wtr_profile_sys_free(prof, fs);
			return NULL;
		}
	}
	ptr += (sizeof(wt_u32)*argc);

	/* copy return types with byte order swap and resolutions */

	fs->retv = p = (wt_u32*)ptr;
	for(i=0;i<retc;++p,++retv,++i)
	{
		*p = wtr_botoh32(*retv, bo);
		if(!wtr_program_resolvetype(prog, p))
		{
			wtr_profile_sys_free(prof, fs);
			return NULL;
		}
	}
	ptr += (sizeof(wt_u32)*retc);

	/* copy function name */

	memcpy(ptr, name, nl);
	fs->name = ptr;
	/*ptr += nl;*/

	return fs;
}

void wtr_funcsig_delete(struct wtr_profile* prof, struct wtr_funcsig* fsig)
{
	if(NULL!=fsig)
	{
		wtr_profile_sys_free(prof, fsig);
	}
}

static int wtr_funcsig_cmp_impl(const struct wtr_funcsig* lhs, const struct wtr_funcsig* rhs, const wt_bool cmp_retv)
{
	const int     nc = strcmp(lhs->name, rhs->name);
	wt_u32        i;
	wt_u32        n;
	const wt_u32* lp;
	const wt_u32* rp;

	if(0!=nc)
	{
		return nc;
	}

	if(lhs==rhs)
	{
		return 0;
	}

	if(lhs->pure<rhs->pure)
	{
		return -1;
	}
	else
	if(lhs->pure>rhs->pure)
	{
		return +1;
	}

	if(lhs->argc<rhs->argc)
	{
		return -1;
	}
	else
	if(lhs->argc>rhs->argc)
	{
		return +1;
	}

	n  = lhs->argc;
	lp = lhs->argv;
	rp = rhs->argv;

	for(i=0;i<n;++lp,++rp,++i)
	{
		wt_u32 lt = *lp;
		wt_u32 rt = *rp;

		if(lt<rt)
		{
			return -1;
		}
		else
		if(lt>rt)
		{
			return +1;
		}
	}

	if(cmp_retv)
	{
		for(i=0,n=lhs->retc,lp=lhs->retv,rp=rhs->retv;i<n;++lp,++rp,++i)
		{
			if((*lp)<(*rp))
			{
				return -1;
			}
			else
			if((*lp)>(*rp))
			{
				return +1;
			}
		}
	}

	return 0;
}

int wtr_funcsig_cmp(const struct wtr_funcsig* lhs, const struct wtr_funcsig* rhs)
{
	return wtr_funcsig_cmp_impl(lhs, rhs, WT_FALSE);
}

int wtr_funcsig_cmp_with_retv(const struct wtr_funcsig* lhs, const struct wtr_funcsig* rhs)
{
	return wtr_funcsig_cmp_impl(lhs, rhs, WT_TRUE);
}
