#ifndef WTRUNTIME_HASH_H
#define WTRUNTIME_HASH_H

#include "config.h"
#include "types.h"
#include "byteorder.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/* Find supported hash algorithm by name. */
/* Returns: algorithm identifier (WTBHA_*); 0, if the hash algorithm is not supported */
/*   name - name hash algorithm */
wt_u16 wtr_hash_find(const char* name);


/* Get the hash algorithm name. */
/* Returns: name of the hash algorithm; NULL, if the hash algorithm is not supported */
/*   hashalg - hash algorithm identifier (WTBHA_*) */
const char* wtr_hash_name(wt_u16 hashalg);


/* Get the hash digest length in octets. */
/* Returns: length of the corresponding hash digest; 0, if the hash algorithm is not supported */
/*   hashalg - hash algorithm identifier (WTBHA_*) */
size_t wtr_hash_len(wt_u16 hashalg);


/* Calculate the hash of a Welltype Binary. */
/* Returns: error code */
/*   out_digest - the digest will be stored in this buffer (the caller must ensure that the digest fits in the buffer) */
/*   hashalg    - hash algorithm identifier (WTBHA_*) */
/*   data       - the Welltype Binary data (must start with the WTB signature) */
/*   size       - length of the 'data' in octets */
wt_error wtr_hash_calc(void* out_digest, wt_u16 hashalg, const void* data, size_t size, wt_bo bo);


/* Checks whether the computed hash matches with the reference hash value. */
/* Returns: error code */
/*   digest  - the reference hash value */
/*   hashalg - hash algorithm identifier (WTBHA_*) */
/*   data    - the Welltype Binary data (must start with the WTB signature) */
/*   size    - length of the 'data' in octets */
wt_error wtr_hash_check(const void* digest, wt_u16 hashalg, const void* data, size_t size, wt_bo bo);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* WTRUNTIME_HASH_H */
