#ifndef WTRUNTIME_TYPES_H
#define WTRUNTIME_TYPES_H

#include "config.h"
#include "wtruntime.h"
#include <stddef.h>

/* Type of the "pointer" used in the virtual machine. This is an offset */
/* based on 'profile->memory'. */
/* Currently this is a 32-bit integer, which limits the maximal addressable */
/* memory to 4 GiB. */
typedef wt_u32 wt_vmptr;

struct wtr_objentry
{
	wt_pfn   pfn;
	wt_vmptr obj;
};

#define WTR_NULLPC  (/*~*/((wt_u32)0))

struct wtr_program;

struct wtr_exception
{
	wt_u32              refcount;
	wt_exception        exception;
	struct wtr_program* owner;
	char                name[];
};

#define wtr_contof(_ptr_, _type_, _field_) \
	((_type_*)(((wt_u8*)(_ptr_)) - (size_t)(&(((_type_*)0)->_field_))))

struct wtr_listnode;

struct wtr_listnode
{
	struct wtr_listnode* next;
	struct wtr_listnode* prev;
};

#define wtr_list wtr_listnode
#define wtr_node wtr_listnode

#define wtr_list_init(_list_) \
	do \
	{ \
		struct wtr_listnode* const list___ = (_list_); \
		list___->next = list___; \
		list___->prev = list___; \
	} \
	while(0)

#define wtr_list_empty(_list_) \
	(((_list_)->next)==(_list_))

#define wtr_list_first(_list_) \
	((_list_)->next)

#define wtr_list_prev(_list_) \
	((_list_)->prev)

#define wtr_list_next(_list_) \
	((_list_)->next)

#define wtr_list_remove(_node_) \
	do \
	{ \
		struct wtr_listnode* const node___ = (_node_); \
		node___->prev->next = node___->next; \
		node___->next->prev = node___->prev; \
	} \
	while(0)

#define wtr_list_insert_head(_list_, _node_) \
	do \
	{ \
		struct wtr_listnode* const list___ = (_list_); \
		struct wtr_listnode* const node___ = (_node_); \
		node___->prev = list___; \
		node___->next = list___->next; \
		list___->next->prev = node___; \
		list___->next = node___; \
	} \
	while(0)

#define wtr_list_insert_tail(_list_, _node_) \
	do \
	{ \
		struct wtr_listnode* const list___ = (_list_); \
		struct wtr_listnode* const node___ = (_node_); \
		node___->next = list___; \
		node___->prev = list___->prev; \
		list___->prev->next = node___; \
		list___->prev = node___; \
	} \
	while(0)

#define wtr_list_for_each(_list_, _node_) \
	for((_node_)=(_list_)->next;(_node_)!=(_list_);(_node_)=(_node_)->next)

#endif /* WTRUNTIME_TYPES_H */
